local Config_PanelList = 
{
    ["Messages"] = "XToLevelConfig_pnlMessages",
    ["Average Window"] = "XToLevelConfig_pnlAverage",
    ["Average Data"] = "XToLevelConfig_pnlAvrgData",
    ["LDB Text"] = "XToLevelConfig_pnlLDBText",
    ["Tooltip"] = "XToLevelConfig_pnlTooltip",
}
local Config_AllowedLocales =
{
    ["English"] = "enUS",
    ["Deutsch"] = "deDE",
    ["Français"] = "frFR",
    ["Español"] = "esES",
    ["Dansk"] = "dkDK",
}
local currentPanel = nil; -- Stores the current panel between Locale swaps.

--
-- Main OnShow
--  Hanles localization and shows the initial panel.
--
---
-- function description
function XToLevelConfig_OnShow()
    if not currentPanel then
        XToLevelConfig_SwapPanels('Messages');
    end
    
    XToLevelConfig_MessageColors_Initialize();
    AverageFrame_ToogleOldBoxes(sConfig.averageDisplay.mode == 2)
    
    -- Disable pet related stuff if no pet is enabled.
    local hasPet = XToLevel.Pet.isActive or XToLevel.Pet.hasBeenActive
    if not hasPet then
        XToLevelConfig_cbShowPetFrame:Disable()
        --XToLevelConfig_cbDetachPetFrame:Disable()
    else
        XToLevelConfig_cbShowPetFrame:Enable()
    end
    
    -- Disable Dungeon and Battleground options for low level players
    if XToLevel.Player.level < 10 then
        XToLevel.Tooltip:SetConfigInfo(XToLevelConfig_cbAverageBattlegrounds, "This option becomes available at level 10")
        XToLevel.Tooltip:SetConfigInfo(XToLevelConfig_cbAveragePlayerBGObjectives, "This option becomes available at level 10")
        XToLevel.Tooltip:SetConfigInfo(XToLevelConfig_cbLDBTextPlayerBattles, "This option becomes available at level 10")
        XToLevel.Tooltip:SetConfigInfo(XToLevelConfig_cbLDBTextPlayerBGObjectives, "This option becomes available at level 10")
        
    end
    if XToLevel.Player.level < 15 then
        XToLevel.Tooltip:SetConfigInfo(XToLevelConfig_cbAverageDungeons, "This option becomes available at level 15")
        XToLevel.Tooltip:SetConfigInfo(XToLevelConfig_cbLDBTextPlayerDungeons, "This option becomes available at level 15")
    end
    
    --
    -- Initialize Popup Windows
    --
    StaticPopupDialogs['XToLevelConfig_MessageColorsReset'] = {
		text = L['Color Reset Dialog'],
		button1 = "Yes",
		button2 = "No",
		OnAccept = function()
			sConfig.messages.colors = {
				playerKill = {0.72, 1, 0.71, 1},
				playerQuest = {0.5, 1, 0.7, 1},
				playerBattleground = {1, 0.5, 0.5, 1},
				playerLevel = {0.35, 1, 0.35, 1},
				petKill = {0.52, 0.73, 1, 1},
			};
			XToLevelConfig_MessageColors_Display();
		end,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
	}
    StaticPopupDialogs['XToLevelConfig_ResetPlayerKills'] = {
		text = L['Reset Player Kill Dialog'],
		button1 = "Yes",
		button2 = "No",
		OnAccept = function()
			XToLevel.Player:ClearKills();
            XToLevel.Average:Update();
            XToLevel.LDB:BuildPattern();
            XToLevel.LDB:Update();
		end,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
	}
    StaticPopupDialogs['XToLevelConfig_ResetPlayerQuests'] = {
		text = L['Reset Player Quest Dialog'],
		button1 = "Yes",
		button2 = "No",
		OnAccept = function()
			XToLevel.Player:ClearQuests();
			XToLevel.Average:Update();
            XToLevel.LDB:BuildPattern();
            XToLevel.LDB:Update();
		end,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
	}
    StaticPopupDialogs['XToLevelConfig_ResetBattlegrounds'] = {
		text = L['Reset Battleground Dialog'],
		button1 = "Yes",
		button2 = "No",
		OnAccept = function()
			XToLevel.Player:ClearBattlegrounds();
			XToLevel.Average:Update();
            XToLevel.LDB:BuildPattern();
            XToLevel.LDB:Update();
		end,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
	}
    StaticPopupDialogs['XToLevelConfig_ResetPetKills'] = {
		text = L['Reset Pet Kill Dialog'],
		button1 = "Yes",
		button2 = "No",
		OnAccept = function()
			XToLevel.Pet:ClearKills();
			XToLevel.Average:Update();
            XToLevel.LDB:BuildPattern();
            XToLevel.LDB:Update();
		end,
		timeout = 0,
		whileDead = true,
		hideOnEscape = true,
	}
	StaticPopupDialogs['XToLevelConfig_ResetDungeons'] = {
        text = L['Reset Dungeon Dialog'],
        button1 = "Yes",
        button2 = "No",
        OnAccept = function()
            XToLevel.Player:ClearDungeonList();
            XToLevel.Average:Update();
            XToLevel.LDB:BuildPattern();
            XToLevel.LDB:Update();
        end,
        timeout = 0,
        whileDead = true,
        hideOnEscape = true,
    }

    --
    -- Localize
    --
    XToLevelConfig_cbDebugText:SetText(L["Show Debug Info"]);
    XToLevelConfig_lblLocationHeaderLabel:SetText(L[currentPanel]);
    
    -- Buttons
    XToLevelConfig_btnTabMessagesText:SetText(L['Messages']);
    XToLevelConfig_btnTabAverageWindowText:SetText(L['Window Tab']);
    XToLevelConfig_btnTabAverageDataText:SetText(L['Data']);
    XToLevelConfig_btnTabLDBText:SetText(L['LDB Tab']);
    XToLevelConfig_btnTabTooltipText:SetText(L['Tooltip']);
    XToLevelConfig_btnCloseText:SetText(L['Close']);
    
    -- Messages
    XToLevelConfig_lblPlayerMessagesLabel:SetText(L["Player Messages"]);
    XToLevelConfig_cbShowPlayerFloatingMessagesText:SetText(L["Show Floating"]);
    XToLevelConfig_cbShowPlayerChatMessagesText:SetText(L["Show In Chat"]);
    XToLevelConfig_cbShowPlayerBGObjectivesText:SetText(L["Show BG Objectives"]);
    XToLevelConfig_lblPetMessagesLabel:SetText(L["Pet Messages"]);
    XToLevelConfig_cbShowPetFloatingMessagesText:SetText(L["Show Floating"]);
    XToLevelConfig_cbShowPetChatMessagesText:SetText(L["Show In Chat"]);
    XToLevelConfig_lblMessageColorsLabel:SetText(L["Message Colors"]);
    
    XToLevelConfig_cpPlayerKillsText:SetText(L['Player Kills']);
    XToLevelConfig_cpPlayerQuestsText:SetText(L['Player Quests']);
    XToLevelConfig_cpPlayerBGsText:SetText(L['Player Battles']);
    XToLevelConfig_cpPlayerLevelText:SetText(L['Player Levelup']);
    XToLevelConfig_cpPetKillsText:SetText(L['Pet Kills']);
    XToLevelConfig_btnMessageColorResetText:SetText(L['Color Reset']);
    
    -- Average Window
    XToLevelConfig_lblAverageAppearenceLabel:SetText(L["Appearence"]);
    -- XToLevelConfig_AverageBehaviorLabel:SetText(L["Behavior"]);
    -- XToLevelConfig_lblAverageDataLabel:SetText(L["Data"]);
    XToLevelConfig_cbAvrgShowHeaderText:SetText(L["Show XToLevel Header"]);
    XToLevelConfig_cbShowAverageFrameText:SetText(L["Show Window Frame"]);
    XToLevelConfig_cbShowAverageTooltipText:SetText(L["Show Tooltip"]);
    XToLevelConfig_cbShowAverageVerboseText:SetText(L["Show Verbose Text"]);
    XToLevelConfig_cbShowAverageColoredText:SetText(L["Show Colored Text"]);
    XToLevelConfig_cbAverageLockText:SetText(L["Lock Avarage Display"]);
    XToLevelConfig_cbAverageAllowConfigText:SetText(L["Allow Average Click"]);
    XToLevelConfig_cbAvrgOrientationText:SetText(L["Vertical Align"]);
    XToLevelConfig_cbShowPetFrameText:SetText(L["Show Pet Frame"]);
    --XToLevelConfig_cbDetachPetFrameText:SetText(L["Detach Pet Frame"]);
    XToLevelConfig_cbAveragePlayerKillsText:SetText(L["Player Kills"]);
    XToLevelConfig_cbAveragePlayerQuestsText:SetText(L["Player Quests"]);
    XToLevelConfig_cbAveragePetKillsText:SetText(L["Pet Kills"]);
    XToLevelConfig_cbAverageBattlegroundsText:SetText(L["Player Battles"]);
    XToLevelConfig_cbAveragePlayerBGObjectivesText:SetText(L["Player Objectives"]);
    XToLevelConfig_cbAverageProgressText:SetText(L['Progress']);
    
    XToLevelConfig_cbAverageProgressAsBarsText:SetText(L['Progress As Bars']);
    XToLevelConfig_cbAveragePetProgressText:SetText(L['Pet Progress']);
    XToLevelConfig_cbShowTooltipCombinedText:SetText(L['Combine Tooltip Data']);
    
    -- Average Data
    XToLevelConfig_lblAvrgDataRangesLabel:SetText(L['Number of records used'] .. ":");
    XToLevelConfig_lblAvrgDataKillsLabel:SetText(L['Kills'] .. ":");
    XToLevelConfig_lblAvrgDataQuestsLabel:SetText(L['Quests'] .. ":");
    XToLevelConfig_lblAvrgDataBattlesLabel:SetText(L['Battles'] .. ":");
    XToLevelConfig_lblAvrgDataObjectivesLabel:SetText(L['Objectives'] .. ":");
    XToLevelConfig_lblAvrgDataPetLabel:SetText(L['Pet'] .. ":");
    
    XToLevelConfig_lblResetDataLabel:SetText(L['Reset Data']);
    XToLevelConfig_btnResetPlayerKillsText:SetText(L['Reset Player Kills']);
    XToLevelConfig_btnResetPlayerQuestsText:SetText(L['Reset Player Quests']);
    XToLevelConfig_btnResetBattlegroundsText:SetText(L['Reset Battlegrounds']);
    XToLevelConfig_btnResetDungeonsText:SetText(L['Reset Dungeons']);
    XToLevelConfig_btnResetPetKillsText:SetText(L['Reset Pet Kills']);
    
    -- LDB Text
    -- XToLevelConfig_lblLDBTextAppearenceLabel:SetText(L['Appearence']);
    XToLevelConfig_cbShowLDBTextText:SetText(L['Show Text']);
    XToLevelConfig_cbShowLDBLabelText:SetText(L['Show Label']);
    XToLevelConfig_cbShowLDBIconText:SetText(L['Show Icon']);
    XToLevelConfig_ShowLDBColorText:SetText(L['Allow Colored Text']);
    XToLevelConfig_cbLDBTextVerboseText:SetText(L['Verbose']);
    XToLevelConfig_cbLDBTextPlayerKillsText:SetText(L['Kills']);
    XToLevelConfig_LDBTextPlayerXPText:SetText(L['XP %']);
    XToLevelConfig_cbLDBTextPlayerQuestsText:SetText(L['Quests']);
    XToLevelConfig_cbLDBTextPlayerDungeonsText:SetText(L['Dungeons']);
    XToLevelConfig_cbLDBTextXPColorsText:SetText(L['Color By XP']);
    XToLevelConfig_cbLDBTextPlayerBattlesText:SetText(L['Battles']);
    XToLevelConfig_cbLDBTextPetKillsText:SetText(L['Pet']);
    XToLevelConfig_cbLDBTextPlayerBGObjectivesText:SetText(L['Objectives']);
    XToLevelConfig_cbLDBTextPetXPText:SetText(L['Pet XP %']);
    
    -- Tooltip
    XToLevel_lblTooltipSectionsLabel:SetText(L['Data']);
    XToLevelConfig_cbTooltipDetailsText:SetText(L['Show Player Details']);
    XToLevelConfig_cbXPDetailsText:SetText(L['Show Player Experience']);
    XToLevelConfig_cbBGDetailsText:SetText(L['Show Battleground Info']);
    XToLevelConfig_cbDungeonDetailsText:SetText(L['Show Dungeon Info']);
    XToLevelConfig_cbTooltipPetInfoText:SetText(L['Show Pet Details']);
end

---
-- function description
function btnClose_OnClick()
    XToLevelConfig:Hide();
end

--
-- Color Pickers
--
---
-- function description
function XToLevelConfig_MessageColors_Initialize()
    local workingColor = {0, 0, 1, nil}; -- Throw-away resource for the functions to use.
    
    -- Set colors
    XToLevelConfig_MessageColors_Display();
    
    -- Set callbacks
    XToLevelConfig_cpPlayerKills.colorChangeCallback = function(self)
        if self.currentColor then
            sConfig.messages.colors.playerKill = self.currentColor
        end
    end
    XToLevelConfig_cpPlayerQuests.colorChangeCallback = function(self)
        if self.currentColor then
            sConfig.messages.colors.playerQuest = self.currentColor
        end
    end
    XToLevelConfig_cpPlayerBGs.colorChangeCallback = function(self)
        if self.currentColor then
            sConfig.messages.colors.playerBattleground = self.currentColor
        end
    end
    XToLevelConfig_cpPlayerLevel.colorChangeCallback = function(self)
        if self.currentColor then
            sConfig.messages.colors.playerLevel = self.currentColor
        end
    end
    XToLevelConfig_cpPetKills.colorChangeCallback = function(self)
        if self.currentColor then
            sConfig.messages.colors.petKill = self.currentColor
        end
    end
end
---
-- function description
function XToLevelConfig_MessageColors_Display()
    XToLevelConfig_cpPlayerKills.currentColor = sConfig.messages.colors.playerKill;
    XToLevelConfig_cpPlayerKillsColor:SetTexture(unpack(sConfig.messages.colors.playerKill));
    XToLevelConfig_cpPlayerQuests.currentColor = sConfig.messages.colors.playerQuest;
    XToLevelConfig_cpPlayerQuestsColor:SetTexture(unpack(sConfig.messages.colors.playerQuest));
    XToLevelConfig_cpPlayerBGs.currentColor = sConfig.messages.colors.playerBattleground;
    XToLevelConfig_cpPlayerBGsColor:SetTexture(unpack(sConfig.messages.colors.playerBattleground));
    XToLevelConfig_cpPlayerLevel.currentColor = sConfig.messages.colors.playerLevel;
    XToLevelConfig_cpPlayerLevelColor:SetTexture(unpack(sConfig.messages.colors.playerLevel));
    XToLevelConfig_cpPetKills.currentColor = sConfig.messages.colors.petKill;
    XToLevelConfig_cpPetKillsColor:SetTexture(unpack(sConfig.messages.colors.petKill));
end

---
-- function description
function XToLevelConfig_MessageColors_Reset()
    sConfig.messages.colors.playerKill =         {0.72, 1, 0.71, 1};
    sConfig.messages.colors.playerQuest =        {0.5, 1, 0.7, 1};
    sConfig.messages.colors.playerBattleground = {1, 0.5, 0.5, 1};
    sConfig.messages.colors.playerLevel =        {0.35, 1, 0.35, 1};
    sConfig.messages.colors.petKill =            {0.52, 0.73, 1, 1};
end

--
-- Locale
--
---
-- function description
function cboLocale_OnShow()
    local localeValue = sConfig.general.displayLocale;
    local languageName = nil;
    for language, locale in pairs(Config_AllowedLocales) do
        if locale == localeValue then
            languageName = language;
        end
    end
    if not languageName then
        languageName = "English";
    end
    UIDropDownMenu_SetSelectedName(XToLevelConfig_cboLocale, languageName, true);
    UIDropDownMenu_SetText(XToLevelConfig_cboLocale, languageName);
end
---
-- function description
function cboLocale_OnSelectChanged()
    local rawLanguage = UIDropDownMenu_GetText(XToLevelConfig_cboLocale);
    local newLocale = nil;
    for lang, locale in pairs(Config_AllowedLocales) do
        if rawLanguage == lang then
            newLocale = locale;
        end
    end
    if newLocale then
        -- Set the locale and update the displays
        LOCALE_DISPLAY = newLocale;
		L = LOCALE[LOCALE_DISPLAY]
        sConfig.general.displayLocale = LOCALE_DISPLAY;
        XToLevel.Average:Update();
        XToLevel.LDB:BuildPattern();
        XToLevel.LDB:Update();
        
        -- Refresh the Config window.
        XToLevelConfig:Hide();
        XToLevelConfig:Show();
        XToLevelConfig_SwapPanels(currentPanel);

        console:log("Display locale set to '".. tostring(newLocale) .. "' (" .. tostring(rawLanguage) ..")");
    else
        console:log("Failed to set display locale. '".. tostring(rawLanguage) .."' is not an recognized locale.");
    end
end

--
-- Swap panels
--
---
-- function description
function XToLevelConfig_SwapPanels(newPanel)
    if not newPanel or not Config_PanelList[newPanel] then
        console:log("Could not swap panes. newPanel is not valid ('" .. tostring(newPanel) .."')");
        return false;
    else
        for name, panel in pairs(Config_PanelList) do
            if name == newPanel then
                _G[panel]:Show();
                _G[panel]:SetPoint("TOPLEFT", 20, -120);
                XToLevelConfig_lblLocationHeaderLabel:SetText(L[newPanel]);
                currentPanel = newPanel;
            else
                _G[panel]:Hide();
            end
        end
    end
end

--
-- Average Data
--
local sliderLastUpdate = nil;
---
-- function description
function AvrgData_SyncSliderAndTextbox(self, isSlider)
    if self ~= nil then
        if isSlider then
            local box = _G[string.gsub(self:GetName(), '_sl', '_tb')];
            if box ~= nil then
                local newValue = tonumber(self:GetValue())
                if newValue ~= nil then
                    box:SetText(newValue);
                else
                    console:log("Failed to sync Slider and Textbox. '" .. tostring(self:GetValue()) .."' is not a valid value.");
                end
            end
        else
            local slider = _G[string.gsub(self:GetName(), '_tb', '_sl')];
            if slider ~= nil then
                local newValue = tonumber(self:GetText());
                if newValue ~= nil then
                    slider:SetValue(newValue);
                else
                    console:log("Failed to sync Slider and Textbox. '" .. tostring(self:GetText()) .."' is not a valid value.");
                end
            end
        end
    end
end
---
-- function description
function AvrgData_UpdateConfig(key, value)
    if sConfig.averageDisplay[key] ~= nil then
        sConfig.averageDisplay[key] = tonumber(value) or 10;
        
        XToLevel.Player.killAverage = nil;
        XToLevel.Player.questAverage = nil;
        XToLevel.Player.bgAverage = nil;
        XToLevel.Player.bgObjAverage = nil;        
        XToLevel.Pet.killAverage = nil;
        
        console:log("'".. tostring(key) .."' set to '" .. tostring(value) .."'.");
        
        XToLevel.Average:Update();
        XToLevel.LDB:BuildPattern();
        XToLevel.LDB:Update();
    else
        console:log("Failed to update Average Data value. Key '" .. tostring(key) .. "' is invalid.");
    end
end

--
-- LDB Callbacks
--
---
-- function description
function cboLDBTextPattern_OnSelectChanged()
    local newPattern = UIDropDownMenu_GetText(XToLevelConfig_cboLDBTextPattern);
    if newPattern ~= sConfig.ldb.textPattern then
        sConfig.ldb.textPattern = newPattern;
    end
    if newPattern == "custom" then
        XToLevelConfig_tbLDBTextCustomPattern:Show();
    else
        XToLevelConfig_tbLDBTextCustomPattern:Hide();
    end
    XToLevel.LDB:BuildPattern();
    XToLevel.LDB:Update();
end

--
-- Average Windows
--
---
-- function description
function cboAverageFrame_OnSelectChanged()
    local textMode = UIDropDownMenu_GetText(XToLevelConfig_cboAverageFrame);
    local newMode = 0
    local oldMode = sConfig.averageDisplay.mode
    
    if textMode == 'Classic' then
        newMode = 2
    elseif textMode == 'Blocky' then
        newMode = 1
    end
    
    AverageFrame_ToogleOldBoxes(newMode == 2)
    
    if newMode == oldMode then
        return true
    elseif newMode == 0 then
        sConfig.averageDisplay.visible = false
        sConfig.averageDisplay.mode = 0
        XToLevel.Average:Update()
        return true
    else
        sConfig.averageDisplay.visible = true
        sConfig.averageDisplay.mode = newMode
        AverageFrame_AlignBoxes(textMode)
        XToLevel.Average:Update()
        return true
    end
end

---
-- function description
function AverageFrame_AlignBoxes(child)
    local new = XToLevel_AverageFrame_Blocky_PlayerFrame
	local old = XToLevel_AverageFrame_Classic
	
    if child == "Classic" then
        local point, relativeTo, relativePoint, xOfs, yOfs = new:GetPoint()
        old:ClearAllPoints()
        old:SetPoint(point, relativeTo, relativePoint, xOfs, yOfs)
    else
        local point, relativeTo, relativePoint, xOfs, yOfs = old:GetPoint()
        new:ClearAllPoints()
        new:SetPoint(point, relativeTo, relativePoint, xOfs, yOfs)
    end
end

---
-- function description
function AverageFrame_ToogleOldBoxes(visible)
    if visible then
        -- Hide "New" config boxes
        XToLevelConfig_cbAvrgOrientation:Hide()
        XToLevelConfig_cbShowPetFrame:Hide()
        
        -- Show "Old" config boxes
        XToLevelConfig_cbAvrgShowHeader:Show()
        XToLevelConfig_cbShowAverageFrame:Show()
        XToLevelConfig_cbShowAverageVerbose:Show()
        XToLevelConfig_cbShowAverageColored:Show()
    else
        -- Hide "Old" config boxes
        XToLevelConfig_cbAvrgShowHeader:Hide()
        XToLevelConfig_cbShowAverageFrame:Hide()
        XToLevelConfig_cbShowAverageVerbose:Hide()
        XToLevelConfig_cbShowAverageColored:Hide()
        
        -- Show "New" config boxes
        XToLevelConfig_cbAvrgOrientation:Show()
        XToLevelConfig_cbShowPetFrame:Show()
    end
end
